<?php

namespace Drupal\multi_crop_embed;

use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Plugin\Field\FieldType\ImageItem;
use Drupal\media\MediaInterface;
use Drupal\multi_crop\MultiCropPluginManager;

/**
 * Service description.
 */
class MultiCropEmbedService {

  /**
   * The plugin.manager.multi_crop service.
   *
   * @var \Drupal\multi_crop\MultiCropPluginManager
   */
  protected $multiCropPluginManager;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Constructs a MultiCropEmbedService object.
   *
   * @param \Drupal\multi_crop\MultiCropPluginManager $multi_crop
   *   The plugin.manager.multi_crop service.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   */
  public function __construct(MultiCropPluginManager $multi_crop, EntityDisplayRepositoryInterface $entity_display_repository, EntityRepositoryInterface $entity_repository) {
    $this->multiCropPluginManager = $multi_crop;
    $this->entityDisplayRepository = $entity_display_repository;
    $this->entityRepository = $entity_repository;
  }

  /**
   * Add Crop widget to a form.
   *
   * @param array $form
   *   Form.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   FormState.
   */
  public function addCropWidget(array &$form, FormStateInterface $form_state) {

    $build_info = $form_state->getBuildInfo();
    /** @var \Drupal\editor\Entity\Editor $editor */
    $editor = $build_info['args'][0];
    /** @var \Drupal\filter\Entity\FilterFormat $filterSettings */
    $filterSettings = $editor->getFilterFormat();

    $settings = $filterSettings->toArray();
    if (!isset($settings['filters']['multi_crop_embed'])) {
      return;
    }

    // Retrieve the source field.
    $media_embed_element = $form_state->get('media_embed_element');

    /** @var \Drupal\media\MediaInterface $media */
    $media = $this->entityRepository->loadEntityByUuid('media', $media_embed_element['data-entity-uuid']);
    $source_field_name = $this->recoverSourceField($media);

    $plugin_settings = $settings['filters']['multi_crop_embed'];

    $plugins = $this->loadCropPlugins($plugin_settings, $media->bundle(), $source_field_name);

    // Load multi crop plugins.
    foreach ($plugins as $plugin_id => $image_style_crops) {
      $plugin = $this->multiCropPluginManager->createInstance($plugin_id);

      $media_library_form_display = $this->entityDisplayRepository->getFormDisplay('media', $media->bundle());

      // Change image source component to load correct widget.
      $component_data = $plugin->getComponentConfig($media_embed_element, $image_style_crops);

      $media_library_form_display->setComponent($source_field_name, $component_data);

      // Render form.
      $widget = $media_library_form_display->getRenderer($source_field_name);
      $items = $media->get($source_field_name);
      $items->filterEmptyItems();

      // Extract widget of source field.
      $form[$source_field_name] = $widget->form($items, $form, $form_state);

      // Finishing form.
      $plugin->finishElement($form, $source_field_name, $media_embed_element);
    }
  }

  /**
   * Gets the name of an image media item's source field.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media item being embedded.
   *
   * @return string|null
   *   The name of the image source field configured for the media item, or
   *   NULL if the source field is not an image field.
   */
  private function recoverSourceField(MediaInterface $media): ?string {
    $field_definition = $media->getSource()
      ->getSourceFieldDefinition($media->bundle->entity);
    $item_class = $field_definition->getItemDefinition()->getClass();
    if (is_a($item_class, ImageItem::class, TRUE)) {
      return $field_definition->getName();
    }
    return NULL;
  }

  /**
   * Find crops needed in media view mode.
   *
   * @param array $settings
   *   Formatter settings.
   * @param string $media_bundle
   *   Media bundle.
   * @param string $source_field
   *   Media source field.
   *
   * @return array
   *   List of crop plugin & settings.
   */
  public function loadCropPlugins(array $settings, $media_bundle, $source_field) {

    $cropPlugins = [];
    $view_mode = $settings['settings']['default_view_mode'];

    // Load view mode of bundle.
    $media_display = $this->entityDisplayRepository->getViewDisplay('media', $media_bundle, $view_mode);
    $field_settings = $media_display->getComponent($source_field);

    // If view mode has an image style.
    if (isset($field_settings['settings']['image_style'])) {
      $image_style_id = $field_settings['settings']['image_style'];

      /** @var \Drupal\image\Entity\ImageStyle $image_style */
      $image_style = $this->entityRepository->loadEntityByConfigTarget('image_style', $image_style_id);
      $effects = $image_style->getEffects()->getConfiguration();
      $plugins = $this->multiCropPluginManager->getDefinitions();

      // Find crops used in image style.
      foreach ($effects as $effect) {
        foreach ($plugins as $plugin) {
          if (in_array($effect['id'], $plugin['image_style_effect'])) {
            if (!isset($cropPlugins[$plugin['id']])) {
              $cropPlugins[$plugin['id']] = [$effect['data']['crop_type']];
            }
            else {
              $cropPlugins[$plugin['id']][] = $effect['data']['crop_type'];
            }
          }
        }
      }
    }

    return $cropPlugins;
  }

}
