<?php

namespace Drupal\multi_crop_embed\Plugin\Filter;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\media\MediaInterface;
use Drupal\media\Plugin\Filter\MediaEmbed;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\multi_crop_embed\MultiCropEmbedService;

/**
 * Provides a filter to embed media items using a custom tag.
 *
 * @Filter(
 *   id = "multi_crop_embed",
 *   title = @Translation("Embed media with Contextual Crop"),
 *   description = @Translation("Add contextual Crop capabilities to Embed Media. Replace Native 'Embed Media' filter."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
 *   settings = {
 *     "default_view_mode" = "default",
 *     "allowed_view_modes" = {},
 *     "allowed_media_types" = {},
 *   },
 *   weight = 98,
 * )
 *
 * @internal
 */
class MultiCropEmbed extends MediaEmbed implements ContainerFactoryPluginInterface, TrustedCallbackInterface {
  /**
   * The current route service.
   *
   * @var \Drupal\multi_crop_embed\MultiCropEmbedService
   */
  protected MultiCropEmbedService $multiCropEmbedService;

  /**
   * Constructs a MultiCropEmbed object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundle_info
   *   The entity type bundle info service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\multi_crop_embed\MultiCropEmbedService $multi_crop_embed_service
   *   The multi_crop_embed service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityRepositoryInterface $entity_repository, EntityTypeManagerInterface $entity_type_manager, EntityDisplayRepositoryInterface $entity_display_repository, EntityTypeBundleInfoInterface $bundle_info, RendererInterface $renderer, LoggerChannelFactoryInterface $logger_factory, MultiCropEmbedService $multi_crop_embed_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_repository, $entity_type_manager, $entity_display_repository, $bundle_info, $renderer, $logger_factory);
    $this->multiCropEmbedService = $multi_crop_embed_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity.repository'),
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('renderer'),
      $container->get('logger.factory'),
      $container->get('multi_crop_embed.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $form['notice'] = [
      '#markup' => $this->t('Please ensure attributes "data-crop-settings" and "data-crop-type" are authorized in the allowed for "drupal-media" HTML tag'),
    ];

    $form = parent::settingsForm($form, $form_state);

    $form['default_view_mode']["#suffix"] = '<div id="multi-crop-plugin-control"></div>';
    $form['default_view_mode']['#ajax'] = [
      'callback' => [$this, 'checkMultiCropPlugin'],
      'event' => 'change',
      'wrapper' => 'multi-crop-plugin-control',
      'progress' => [
        'type' => 'throbber',
        'message' => $this->t('Verifying entry...'),
      ],
    ];
    return $form;
  }

  /**
   * Check compatible MultiCrop Widget.
   *
   * @param array $form
   *   Form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form_state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Print Feedback.
   */
  public function checkMultiCropPlugin(array &$form, FormStateInterface $form_state) {

    $message = [];
    $values = $form_state->getValue('filters');

    if (isset($values['multi_crop_embed']['settings'])) {
      $media_bundle = $values['multi_crop_embed']['settings']['allowed_media_types'];

      foreach ($media_bundle as $bundle => $status) {
        if ($bundle != $status) {
          continue;
        }

        $media_bundle = $this->entityTypeManager->getStorage('media_type')->load($bundle);

        $source_configuration = $media_bundle->getSource()->getConfiguration();

        if (!isset($source_configuration['source_field'])) {
          continue;
        }

        $plugin_list = $this->multiCropEmbedService->loadCropPlugins($values['multi_crop_embed'], $bundle, $source_configuration['source_field']);
        if (count($plugin_list) == 0) {
          $message[] = $this->t("No corresponding Media Contextual Crop Plugin detected.");
          $message[] = $this->t("This may have 2 explanation :");
          $message[] = $this->t("- The view mode don't use crop");
          $message[] = $this->t("- The corresponding Media Contextual Cropping Plugin are not activated.");
        }
        else {
          foreach ($plugin_list as $plugin => $formats) {
            $message[] = $this->t('The @plugin contextual Crop will be used with the @format image style.',
            [
              '@plugin' => $plugin,
              '@format' => implode(', ', $formats),
            ]);
          }

        }
      }

      $response = new AjaxResponse();
      $response->addCommand(
        new ReplaceCommand(
          '#multi-crop-plugin-control',
          '<div id="multi-crop-plugin-control">' . implode('<br />', $message) . '</div>'),
      );
      return $response;
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function applyPerEmbedMediaOverrides(\DOMElement $node, MediaInterface $media) {
    parent::applyPerEmbedMediaOverrides($node, $media);
    if ($image_field = $this->getMediaImageSourceField($media)) {
      if ($node->hasAttribute('data-crop-settings')) {
        $media->{$image_field}->multi_crop_embed_settings = $node->getAttribute('data-crop-settings');
        $media->{$image_field}->multi_crop_plugin_type = $node->getAttribute('data-crop-type');
        $node->removeAttribute('data-crop-settings');
      }
    }
  }

}
