<?php

namespace Drupal\multi_crop\Commands;

use Drush\Commands\DrushCommands;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Config\CachedStorage;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\ProxyClass\Extension\ModuleInstaller;
use Drupal\Core\Entity\EntityTypeManager;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class MultiCropCommands extends DrushCommands {

  /**
   * Module Transformation matrix.
   *
   * @var string[]
   */
  private $moduleAssociation = [
    'multi_crop' => 'media_contextual_crop',
    'multi_crop_embed' => 'media_contextual_crop_embed',
    'multi_crop_field_formatter' => 'media_contextual_crop_field_formatter',
    'multi_crop_focal_point' => 'media_contextual_crop_fp_adapter',
    'multi_crop_image_widget_crop' => 'media_contextual_crop_iwc_adapter',
  ];

  /**
   * Instance of module_handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  private $moduleHandlerService;

  /**
   * Instance of extension.list.module service.
   *
   * @var \Drupal\Core\Extension\ExtensionList
   */
  private $extensionService;

  /**
   * Instance of config.factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  private $configFactory;

  /**
   * Instance of config.storage service.
   *
   * @var \Drupal\Core\Config\CachedStorage
   */
  private $configStorage;

  /**
   * Instance of module_installer service.
   *
   * @var \Drupal\Core\ProxyClass\Extension\ModuleInstaller
   */
  private $moduleInstaller;

  /**
   * Instance of entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  private $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(ModuleHandler $module_handler_service,
                              ExtensionList $extension_service,
                              ModuleInstaller $module_installer_service,
                              ConfigFactory $config_factory_service,
                              CachedStorage $config_storage_service,
                              EntityTypeManager $entity_type_manager_service) {
    $this->moduleHandlerService = $module_handler_service;
    $this->extensionService = $extension_service;
    $this->moduleInstaller = $module_installer_service;
    $this->configFactory = $config_factory_service;
    $this->configStorage = $config_storage_service;
    $this->entityTypeManager = $entity_type_manager_service;
  }

  /**
   * Replace multi-crop to media-contextual-crop collection.
   *
   * @usage multi_crop-migrate-to-MediaContextualCrop
   *   Usage description
   *
   * @command multi_crop:migrateToMediaContextualCrop
   */
  public function migrateCommand() {

    drupal_flush_all_caches();

    // DEPENDENCIES CHECK.
    $this->say(dt('-- Dependencies Check 1/X --'));
    $dependencies_needed = $this->checkDependencies();
    if (count($dependencies_needed['sources_needed']) > 0) {
      $this->logger()->error(dt('Extensions needed '));
      $this->logger()
        ->notice(dt('Please add %extension to your composer ', ['%extension' => implode(', ', $dependencies_needed['sources_needed'])]));
      $composer_require_queue = implode(' drupal/', $dependencies_needed['sources_needed']);
      $this->logger()
        ->notice(dt('by running the command : `composer require drupal/%list `', ['%list' => $composer_require_queue]));
      $this->logger()->notice(dt('and restart command after.'));
      return;
    }

    // DATABASE BACK-UP.
    $this->say(dt('-- Database Back-up 2/X --'));
    $back_up = $this->ask(dt('Did you have make a full database back-up ? [Y-N]'));
    if ($back_up != 'Y') {
      $this->say(dt('Please do a database back-up. The command `drush sql-dump > my_back-up.sql` can do it for you...'));
      $q1 = $this->ask(dt('Did you have make a full database back-up ? [Y-N]'));
      if ($q1 != 'Y') {
        $q2 = $this->ask(dt("Are you really sure you don't need it ? [Y-N]"));
        if ($q2 != 'Y') {
          $this->say(dt("As you wish... "));
        }
      }
    }

    // INSTALL NEW MODULES.
    $this->say(dt('-- Enable new modules 3/X --'));
    // Install CORE API module first.
    $this->moduleInstaller->install(['media_contextual_crop']);
    // Install Other after.
    $this->moduleInstaller->install($dependencies_needed['to_activate']);

    drupal_flush_all_caches();

    // UPDATE CONFIGURATIONS.
    $this->say(dt('-- Update configurations 5/X --'));

    if (in_array('multi_crop_embed', $dependencies_needed['old'])) {
      $this->say(dt('---- Update configurations from multi_crop_embed ---'));
      $this->migrateEmbedConf();

      // Force plugin cache to be rebuild.
      $this->moduleInstaller->install(['media_contextual_crop_embed']);
      $this->moduleInstaller->uninstall(['multi_crop_embed'], FALSE);

    }

    if (in_array('multi_crop_field_formatter', $dependencies_needed['old'])) {
      $this->say(dt('---- Update configurations from multi_crop_field_formatter ---'));
      $this->migrateFieldFormaterConf();

      // Force plugin cache to be rebuild.
      $this->moduleInstaller->install(['media_contextual_crop_field_formatter']);
      $this->moduleInstaller->uninstall(['multi_crop_field_formatter'], FALSE);
    }

    drupal_flush_all_caches();
    // DISABLED multi_crop.
    $this->say(dt('-- Disabled old modules 6/X --'));
    $this->moduleInstaller->uninstall($dependencies_needed['old'], FALSE);

    // END.
    $this->say(dt('-- Finish --'));
    $this->say(dt('Please ensure to flush cache from Back-office and test all features before saving changes'));
    $this->say(dt('please run `composer remove drupal/multi_crop` command for cleaning sources'));

  }

  /**
   * Check Media Contextual Crop module needed.
   *
   * @return array
   *   Dependencies data.
   */
  private function checkDependencies() {
    $active_modules = array_keys($this->moduleHandlerService->getModuleList());
    $multi_crop_active_collection = array_filter(
      $active_modules,
      [$this, 'getMultiCrop']
    );
    $needed = [
      'old' => $multi_crop_active_collection,
      'sources_needed' => [],
      'to_activate' => [],
    ];
    foreach ($multi_crop_active_collection as $module_name) {
      if (!$this->extensionService->exists($this->moduleAssociation[$module_name])) {
        $needed['sources_needed'][] = $this->moduleAssociation[$module_name];
      }
      $needed['to_activate'][] = $this->moduleAssociation[$module_name];
    }

    return $needed;
  }

  /**
   * Check if module is in multi_crop collection.
   *
   * @param string $module_name
   *   Module name.
   *
   * @return bool
   *   The module are in multi_crop collection.
   */
  public function getMultiCrop($module_name) {
    return strstr($module_name, 'multi_crop') !== FALSE;
  }

  /**
   * Migrate configuration files form field formater feature.
   */
  private function migrateFieldFormaterConf() {
    $target = 'multi_crop_field_formatter';
    $configNames = $this->configStorage->listAll('core.entity_view_display');
    foreach ($configNames as $config_name) {
      $conf = $this->configFactory->getEditable($config_name);
      $dependencies = $conf->get('dependencies');
      if (isset($dependencies['module']) && in_array($target, $dependencies['module'])) {
        $this->say('------ Configuration upadated : ' . $config_name);

        // Change module dependencie.
        $key = array_search($target, $dependencies['module']);
        $new_module_dependencies = array_replace($dependencies['module'], [$key => $this->moduleAssociation[$target]]);
        $new_dependencies = $dependencies;
        $new_dependencies['module'] = $new_module_dependencies;
        $conf->set('dependencies', $new_dependencies);

        // Change formater id.
        $content = $conf->get('content');
        foreach ($content as $field_name => $field_data) {
          if ($field_data['type'] == 'image_multicrop') {
            $content[$field_name]['type'] = 'media_contextual_crop_image';
            $conf->set('content', $content);
          }
        }

        // Save config.
        $conf->save();

        // Update entity data;.
        $entity_name = str_replace('core.entity_view_display.', '', $config_name);
        $storage = $this->entityTypeManager->getStorage('entity_view_display');
        $view_display = $storage->load($entity_name);
        $view_display->save();
      }
    }
  }

  /**
   * Migrate configuration files form embed feature.
   */
  private function migrateEmbedConf() {
    $target = 'multi_crop_embed';
    $configNames = $this->configStorage->listAll('filter.format');
    foreach ($configNames as $config_name) {
      $conf = $this->configFactory->getEditable($config_name);
      $dependencies = $conf->get('dependencies');
      if (isset($dependencies['module']) && in_array($target, $dependencies['module'])) {
        $this->say('------ Configuration upadated : ' . $config_name);

        // Change module dependencie.
        $key = array_search($target, $dependencies['module']);
        $new_module_dependencies = array_replace($dependencies['module'], [$key => $this->moduleAssociation[$target]]);
        $new_dependencies = $dependencies;
        $new_dependencies['module'] = $new_module_dependencies;
        $conf->set('dependencies', $new_dependencies);

        // Change filter id.
        $content = $conf->get('filters');
        $media_embed_weight = 0;
        foreach ($content as $filter_name => $filter_data) {
          // Preserve Media Embed weight.
          if ($filter_name == 'media_embed') {
            $media_embed_weight = $filter_data['weight'];
          }
          // Uptate embed crop data.
          if ($filter_name == 'multi_crop_embed') {
            unset($content['multi_crop_embed']);
            $filter_data['id'] = 'media_contextual_crop_embed';
            $filter_data['provider'] = 'media_contextual_crop_embed';
            $content['media_contextual_crop_embed'] = $filter_data;
          }
          // For embed crop to be BEFORE media_embed.
          $content['media_contextual_crop_embed']['weight'] = $media_embed_weight - 1;
          // Set modified conf.
          $conf->set('filters', $content);
        }
        // Save config.
        $conf->save();

        // Update entity data;.
        $entity_name = str_replace('filter.format.', '', $config_name);
        $storage = $this->entityTypeManager->getStorage('filter_format');
        $filter = $storage->load($entity_name);
        $filter->save();
      }
    }
  }

}
